function samples = read_lung(fname)
% reads and preprocesses raw input data

% domain: pulmonary (lung) diseases -- bronchitis, tuberculosis, cancer, their symptoms and risk factors
% samples: each sample corresponds to single object (patient)

% variables (network nodes):
% ARExposure (ARExpozice) -- long term exposure to radon and/or asbestos (both asbestos radon none)
% Bronchitis (prudusky) -- bronchitis disease (yes no)
% Cancer (rakovina plic) -- lung cancer (yes no)
% Dyspnea (dusnost) -- shortness of breath, the sensation of frightening as not being able to get enough air (frequent seldom never)
% PostSovStates (PostsovRep) -- Post-Soviet states visited in past 3 years (yes no)
% Smoking -- measure of tobacco/nicotine exposure (extreme strong medium low none)
% SEAsia (JVAsie) -- Southeast Asia visited in past 3 years (yes no)
% SSAfrica(SubAfrika) -- Sub-Saharan Africa visited in past 3 years (yes no)
% TBC -- tuberculosis disease (yes no)
% TBCVaccination (TBCOckovani) -- preventive vaccination against tuberculosis (yes no)
% XRay (RTGNalez) -- a positive lung X-ray examination finding (yes no)

% the variables were collected in the alphabetical order
N=11;
ARExposure = 1; Bronchitis = 2; Cancer = 3; Dyspnea = 4; PostSovStates = 5; Smoking = 6; SEAsia = 7; SSAfrica = 8; TBC = 9; TBCVaccination = 10; XRay = 11;
 
% node names and sizes
names = {'ARExposure', 'Bronchitis', 'Cancer', 'Dyspnea', 'PostSovStates', 'Smoking', 'SEAsia', 'SSAfrica', 'TBC', 'TBCVaccination', 'XRay'};
node_sizes = [4 2 2 3 2 5 2 2 2 2 2];

% read and preprocess the input data
fid = fopen(fname);
data_raw = textscan(fid,'%s %s %s %s %s %s %s %s %s %s %s','delimiter',',');
nsamples = size(data_raw{1,1},1);

% conversion between the text and numerical representation
for i = 1 : nsamples
    for j=[Bronchitis Cancer PostSovStates SEAsia SSAfrica TBC TBCVaccination XRay]
        switch data_raw{1,j}{i,1}
            case 'yes',          data_raw{1,j}{i,1}=1;
            case 'no',           data_raw{1,j}{i,1}=2;
            case 'NaN',          data_raw{1,j}{i,1}=NaN;
        end
    end
    switch data_raw{1,ARExposure}{i,1}
        case 'both',             data_raw{1,ARExposure}{i,1}=1;
        case 'asbestos',         data_raw{1,ARExposure}{i,1}=2;
        case 'radon',            data_raw{1,ARExposure}{i,1}=3;
        case 'none',             data_raw{1,ARExposure}{i,1}=4;
        case 'NaN',              data_raw{1,ARExposure}{i,1}=NaN;
    end
    switch data_raw{1,Dyspnea}{i,1}
        case 'frequent',         data_raw{1,Dyspnea}{i,1}=1;
        case 'seldom',           data_raw{1,Dyspnea}{i,1}=2;
        case 'never',            data_raw{1,Dyspnea}{i,1}=3;
        case 'NaN',              data_raw{1,Dyspnea}{i,1}=NaN;
    end
    switch data_raw{1,Smoking}{i,1}
        case 'extreme',          data_raw{1,Smoking}{i,1}=1;
        case 'strong',           data_raw{1,Smoking}{i,1}=2;
        case 'medium',           data_raw{1,Smoking}{i,1}=3;
        case 'low',              data_raw{1,Smoking}{i,1}=4;
        case 'none',             data_raw{1,Smoking}{i,1}=5;
        case 'NaN',              data_raw{1,Smoking}{i,1}=NaN;
    end
end

% transform samples into the cell form
samples = cell(N, nsamples);
for i = 1:N
    samples(i,:) = data_raw{1,i}';
end

% replace NaNs by []
% interchangeable in most BN functions 
% for i=1:N
%    for j=1:nsamples
%        if isnan(samples{i,j})
%            samples{i,j}=[];
%        end
%    end
% end
